package com.contextspace;

import static org.junit.Assert.*;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.persistence.OneToMany;
import javax.persistence.Persistence;
import javax.persistence.Table;
import javax.persistence.TableGenerator;
import javax.persistence.TypedQuery;
import javax.persistence.Version;

import org.junit.Test;


public class TestLazyLoading {

    @Test
    public void test() throws Exception {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb:$objectdb/db/test.tmp;drop");
        EntityManager em = emf.createEntityManager();
        
        PDI testPDI = new PDI();
        testPDI.setPdiId("testPDI");
        testPDI.setFirstname("Joe");
        testPDI.setLastname("Bloggs");
        
        SimpleDateFormat df = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss Z");
        testPDI.addLocation(new Location(1.1, 8.1, df.parse("10-08-2012 10:15:00 +1100"), "location1"));
        testPDI.addLocation(new Location(2.1, 7.1, df.parse("11-08-2012 11:15:00 +1100"), "location2"));
        testPDI.addLocation(new Location(3.1, 6.1, df.parse("12-08-2012 12:15:00 +1100"), "location3"));
        testPDI.addLocation(new Location(4.1, 5.1, df.parse("13-08-2012 13:15:00 +1100"), "location4"));
        testPDI.addLocation(new Location(5.1, 4.1, df.parse("14-08-2012 14:15:00 +1100"), "location5"));
        testPDI.addLocation(new Location(6.1, 3.1, df.parse("15-08-2012 15:15:00 +1100"), "location6"));
        testPDI.addLocation(new Location(7.1, 2.1, df.parse("16-08-2012 16:15:00 +1100"), "location7"));
        testPDI.addLocation(new Location(8.1, 1.1, df.parse("17-08-2012 17:15:00 +1100"), "location8"));
        
        em.getTransaction().begin();
        em.persist(testPDI);
        em.getTransaction().commit();
        long id = testPDI.getId();
        em.flush();
        em.close();
        
        em = emf.createEntityManager();
        testPDI = em.find(PDI.class, id);
        em.close();
        emf.close();
        
        System.out.println(testPDI.getLocationList().size());
        System.out.println(testPDI.getLocationList().get(0).getTag());
        

        
    }

    @Table
    @Entity
    public static class PDI extends IDHolder {

        private String pdiId = null;
        private String firstname = null;
        private String lastname = null;
        private List<Location> locationList = new ArrayList<Location>();
        
        public PDI() {
            
        }
        
        /**
         * @return the pdiId
         */
        public String getPdiId() {
            return pdiId;
        }

        /**
         * @param pdiId the pdiId to set
         */
        public void setPdiId(String pdiId) {
            this.pdiId = pdiId;
        }

        /**
         * @return the firstname
         */
        public String getFirstname() {
            return firstname;
        }

        /**
         * @param firstname the firstname to set
         */
        public void setFirstname(String firstname) {
            this.firstname = firstname;
        }

        /**
         * @return the lastname
         */
        public String getLastname() {
            return lastname;
        }

        /**
         * @param lastname the lastname to set
         */
        public void setLastname(String lastname) {
            this.lastname = lastname;
        }

        @OneToMany(targetEntity = Location.class, mappedBy = "owningObject", cascade = CascadeType.ALL, fetch = FetchType.LAZY, orphanRemoval=true)
        public List<Location> getLocationList() {
            return locationList;
        }

        public void addLocation(Location location) {
            if(locationList==null){
                locationList=new ArrayList<Location>();
            }
            location.setOwningObject(this);
            locationList.add(location);
        }

        public void setLocationList(List<Location> locationList) {
            for(Location location : locationList) {
                location.setOwningObject(this);
            }
            this.locationList = locationList;
        }
        
    }
    
    @MappedSuperclass
    @Inheritance(strategy=InheritanceType.TABLE_PER_CLASS)
    @TableGenerator(name="IdTableGen", initialValue=1, allocationSize=50)
    public static abstract class IDHolder implements Serializable{
        private static final long serialVersionUID = 1L;
        

        private long version;

        private long id;

        /**
         * @return the object id
         */
        @Id
        @GeneratedValue(strategy=GenerationType.TABLE, generator="IdTableGen")
        public long getId() {
            return id;
        }

        /**
         * @param id the id to be set
         */
        public void setId(long id) {
            this.id = id;
        }

        @Version
        public long getVersion() {
            return version;
        }
        
        
    }   
   
    @Entity
    public static class Location {
        private double latitude;
        private double longitude;
        private String tag;
        private Date time;
        
        private PDI owningObject;
        
        public Location(){
            
        }
        
        public Location(double latitude, double longitude) {
            super();
            this.latitude = latitude;
            this.longitude = longitude;
            this.time = new Date();
        }
        
        public Location(double latitude, double longitude, String tag) {
            super();
            this.latitude = latitude;
            this.longitude = longitude;
            this.time = new Date();
            this.tag = tag;
        }

        public Location(double latitude, double longitude, Date time) {
            super();
            this.latitude = latitude;
            this.longitude = longitude;
            this.time = time;
        }
        
        public Location(double latitude, double longitude, Date time, String tag) {
            super();
            this.latitude = latitude;
            this.longitude = longitude;
            this.time = time;
            this.tag = tag;
        }

        /**
         * @return the latitude
         */
        public double getLatitude() {
            return latitude;
        }

        /**
         * @param latitude the latitude to set
         */
        public void setLatitude(double latitude) {
            this.latitude = latitude;
        }

        /**
         * @return the longitude
         */
        public double getLongitude() {
            return longitude;
        }

        /**
         * @param longitude the longitude to set
         */
        public void setLongitude(double longitude) {
            this.longitude = longitude;
        }

        /**
         * @return the tag
         */
        public String getTag() {
            return tag;
        }

        /**
         * @param tag the tag to set
         */
        public void setTag(String tag) {
            this.tag = tag;
        }

        /**
         * @return the time
         */
        public Date getTime() {
            return time;
        }

        /**
         * @param time the time to set
         */
        public void setTime(Date time) {
            this.time = time;
        }

        /**
         * @return the owningObject
         */
        @ManyToOne
        public PDI getOwningObject() {
            return owningObject;
        }

        /**
         * @param owningObject the owningObject to set
         */
        public void setOwningObject(PDI owningObject) {
            this.owningObject = owningObject;
        }
    }

}